@@ -39,6 +39,25 @@ class glue_solve_gen
 
 
 
+class glue_solve_tri_default
+  {
+  public:
+  
+  template<typename T1, typename T2>
+  struct traits
+    {
+    static const bool is_row  = false;
+    static const bool is_col  = T2::is_col;
+    static const bool is_xvec = false;
+    };
+  
+  template<typename T1, typename T2> inline static void apply(Mat<typename T1::elem_type>& out, const Glue<T1,T2,glue_solve_tri_default>& X);
+  
+  template<typename eT, typename T1, typename T2> inline static bool apply(Mat<eT>& out, const Base<eT,T1>& A_expr, const Base<eT,T2>& B_expr, const uword flags);
+  };
+
+
+
 class glue_solve_tri
   {
   public:
