@@ -18,30 +18,7 @@
 //! @{
 
 
-//! immediate inverse of a matrix, storing the result in a dense matrix
-template<typename eT>
-inline
-void
-op_inv::apply(Mat<eT>& out, const Mat<eT>& A)
-  {
-  arma_extra_debug_sigprint();
-  
-  // no need to check for aliasing, due to:
-  // - auxlib::inv() copies A to out before inversion
-  // - for 2x2 and 3x3 matrices the code is alias safe
-  
-  bool status = auxlib::inv(out, A);
-  
-  if(status == false)
-    {
-    out.soft_reset();
-    arma_stop_runtime_error("inv(): matrix seems singular");
-    }
-  }
 
-
-
-//! immediate inverse of T1, storing the result in a dense matrix
 template<typename T1>
 inline
 void
@@ -49,19 +26,61 @@ op_inv::apply(Mat<typename T1::elem_type>& out, const Op<T1,op_inv>& X)
   {
   arma_extra_debug_sigprint();
   
+  typedef typename T1::elem_type eT;
+  
   const strip_diagmat<T1> strip(X.m);
   
-  bool status;
-  
-  if(strip.do_diagmat == true)
+  if(strip.do_diagmat)
     {
-    status = op_inv::apply_diagmat(out, strip.M);
+    op_inv::apply_diagmat(out, strip.M);
     }
   else
     {
-    status = auxlib::inv(out, X.m);
-    }
+    const quasi_unwrap<T1> U(X.m);
     
+    if(U.is_alias(out))
+      {
+      Mat<eT> tmp;
+      
+      op_inv::apply_noalias(tmp, U.M);
+      
+      out.steal_mem(tmp);
+      }
+    else
+      {
+      op_inv::apply_noalias(out, U.M);
+      }
+    }
+  }
+
+
+
+template<typename eT>
+inline
+void
+op_inv::apply_noalias(Mat<eT>& out, const Mat<eT>& A)
+  {
+  arma_extra_debug_sigprint();
+  
+  arma_debug_check( (A.n_rows != A.n_cols), "inv(): given matrix must be square sized" );
+  
+  bool status = false;
+  
+  if(A.n_rows <= 4)
+    {
+    status = auxlib::inv_tiny(out, A);
+    }
+  else
+  if(sympd_helper::guess_sympd(A))
+    {
+    status = auxlib::inv_sympd(out, A);
+    }
+  
+  if(status == false)
+    {
+    status = auxlib::inv(out, A);
+    }
+  
   if(status == false)
     {
     out.soft_reset();
@@ -73,7 +92,7 @@ op_inv::apply(Mat<typename T1::elem_type>& out, const Op<T1,op_inv>& X)
 
 template<typename T1>
 inline
-bool
+void
 op_inv::apply_diagmat(Mat<typename T1::elem_type>& out, const T1& X)
   {
   arma_extra_debug_sigprint();
@@ -117,12 +136,15 @@ op_inv::apply_diagmat(Mat<typename T1::elem_type>& out, const T1& X)
     out.steal_mem(tmp);
     }
   
-  return status;
+  if(status == false)
+    {
+    out.soft_reset();
+    arma_stop_runtime_error("inv(): matrix seems singular");
+    }
   }
 
 
 
-//! inverse of T1 (triangular matrices)
 template<typename T1>
 inline
 void
@@ -141,7 +163,6 @@ op_inv_tr::apply(Mat<typename T1::elem_type>& out, const Op<T1,op_inv_tr>& X)
 
 
 
-//! inverse of T1 (symmetric positive definite matrices)
 template<typename T1>
 inline
 void
